'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _walker = require('./walker');

var _walker2 = _interopRequireDefault(_walker);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function walkAttributes(attrs, adapter) {
  return attrs.map(function (attr) {
    switch (attr.type) {
      case 'keyValue':
        return adapter.keyValue(attr);
      case 'singleKey':
        return adapter.singleKey(attr);
      case 'singleValue':
        return adapter.singleValue(attr);
      case 'handlebars':
        return walkHandlebars(attr, adapter, true);
      default:
        return '';
    }
  });
}

function walkChildren(children, adapter) {
  return children.map(function (node) {
    return walkAstNode(node, adapter);
  });
}

function walkFunction(nodes, adapter, subType) {
  if (subType.substr(0, 4) === 'ATTR') {
    return walkAttributes(nodes, adapter);
  } else {
    return walk(nodes, adapter);
  }
}

function walkHandlebars(node, adapter, nowrap) {
  switch (node.subType) {
    case 'IF':
    case 'ATTRIF':
      node.ifBody = walkFunction(node.ifBody, adapter, node.subType);
      node.elseBody = walkFunction(node.elseBody, adapter, node.subType);
      node.condition = walkHandlebars(node.condition, adapter, false);
      return adapter.handlebars.IF(node);

    case 'EACH':
    case 'ATTREACH':
      node.eachBody = walkFunction(node.eachBody, adapter, node.subType);
      node.elseBody = walkFunction(node.elseBody, adapter, node.subType);
      node.iterator = walkHandlebars(node.iterator, adapter, false);
      return adapter.handlebars.EACH(node);

    case 'UNLESS':
    case 'ATTRUNLESS':
      node.unlessBody = walkFunction(node.unlessBody, adapter, node.subType);
      node.elseBody = walkFunction(node.elseBody, adapter, node.subType);
      node.condition = walkHandlebars(node.condition, adapter, false);
      return adapter.handlebars.UNLESS(node);

    case 'SAFEEVALUATION':
      return adapter.handlebars.SAFEEVALUATION(node);

    case 'SINGLEPARTIALS':
      node.parameters = node.parameters.map(function (param) {
        return walkHandlebars(param, adapter, true);
      });
      return adapter.handlebars.SINGLEPARTIALS(node);

    case 'PARTIAL':
      node.children = walk(node.children, adapter);
      node.parameters = node.parameters.map(function (param) {
        return walkHandlebars(param, adapter, true);
      });
      return adapter.handlebars.PARTIAL(node);

    default:
      if (adapter.handlebars[node.subType]) {
        return adapter.handlebars[node.subType](node, nowrap);
      }
      return '';
  }
}

function walkAstNode(node, adapter) {
  switch (node.type) {
    case 'htmlBlockNode':
      var nodeBlock = node;
      return adapter.htmlBlockNode(_extends({}, nodeBlock, {
        attributes: walkAttributes(nodeBlock.attributes, adapter),
        children: walkChildren(nodeBlock.children, adapter)
      }));
    case 'htmlSingleNode':
      var nodeSingle = node;
      return adapter.htmlSingleNode(_extends({}, nodeSingle, {
        attributes: walkAttributes(nodeSingle.attributes, adapter)
      }));
    case 'htmlComment':
    case 'text':
      var nodeSimple = node;
      return adapter[nodeSimple.type](nodeSimple);
    case 'handlebars':
      return walkHandlebars(node, adapter, true);
    default:
      return '';
  }
}

function walk(ast, adapter) {
  return ast.map(function (node) {
    return walkAstNode(node, adapter);
  });
}

function walkTop(name, ast, adapter) {
  var statements = walk(ast, adapter);
  return adapter.topLevel({ name: name, type: 'topLevel', statements: statements });
}

module.exports = walkTop;